/**************************************************************************************
 DEATH TIME - PURE VERILOG
 Top file
 Date	28/06/2014
 Target	DE0 NANO
 --------------------------------------------------------------------------------------
 OBIETTIVO:
 Realizzare un blocco che, leggendo i segnalo pwm P e N puri, fa in modo di ritardare
 l'accensione del segnale che prima era spento, mentre quello che si deve spegnere
 passa inalterato.
 --------------------------------------------------------------------------------------
 NOTE IMPLEMENTATIVE:
 - il clock atteso in ingresso è da 100MHz (ma non è necessario)
 - il deathTime è un parametro espresso in multipli di "clock", ossia di 10ns
 **************************************************************************************/

module DeadTime
(	input	clk,									// clock atteso: 100MHz
	input [10:0] dt,							// deadtime in decine di microsecondi, es: 350 sono 3.5us
	input	p, n,									// segnali del positivo e negativo
	output	P, N);							// segnali reali offsettati

reg [10:0]	Pcnt = 11'd0,
				Ncnt = 11'd0;					// Contatori per il dead time di P e N
reg p_ = 0;
reg n_ = 0;

always @(posedge clk)
begin
	// Gestione di P
	// -------------
	if(!p)
		Pcnt <= 11'd0;
	else
	if(Pcnt < dt)
		Pcnt <= Pcnt + 1'b1;
	
	// Gestione di N
	// -------------
	if(!n)
		Ncnt <= 11'd0;
	else
	if(Ncnt < dt)
		Ncnt <= Ncnt + 1'b1;

	// Per la serie: devo clockare tutto veramente...
	p_ = p && Pcnt >= dt;		// metto "=" apposta e non "<=" per non rischiare di parallelizzare le istruzioni
	n_ = n && Ncnt >= dt;		// metto "=" apposta e non "<=" per non rischiare di parallelizzare le istruzioni
end

// ASSEGNAZIONI
assign P = p_;
assign N = n_;

endmodule
