/**************************************************************************************
 VIRTUAL JTAG 02 - VERILOG
 Top file
 Date	20/10/2014
 Target	DE0 NANO
 --------------------------------------------------------------------------------------
 OBIETTIVO:
 - Creare un esempio di vJTAG per il Linux Day 2014
 --------------------------------------------------------------------------------------
 RISULTATO:
 --------------------------------------------------------------------------------------
 CONCUSIONE:
 --------------------------------------------------------------------------------------
 NOTA DI COMPILAZIONE: vedi progetti LedBlink_Verilog_puro01 e LedBlink_Verilog_puro02.
 Forse potrebbe anche essere diveso, ma occorre editare dei file di progetto (vedi link
	http://quartushelp.altera.com/12.0/mergedProjects/global/pjn/pjn_tab_general.htm
 che indica la modificadi file EDF)
 --------------------------------------------------------------------------------------
 PIN ASSIGMENTS:
	vedi povgetti LedBlink_Verilog_puro01 e LedBlink_Verilog_puro02.
 --------------------------------------------------------------------------------------
 TIME QUEST:
 Come al solito questo bastardo mi da una valanga di warning, soprattutto a causa del
 vJTAG. Per eliminare nuona parte dei warning critici è stato necessario utilizzare il
 comando <drive_clock_uncertainty>. Un esempio e documentazione si trova qui:
 http://quartushelp.altera.com/12.1/mergedProjects/tafs/tafs/tcl_pkg_sdc_ext_ver_1.0_cmd_derive_clock_uncertainty.htm
 Ma attenzione: nell'esempio viene dato un clock troppo alto per il componente in uso;
 viene settato infatti "1!, ma nel mio file .SDC la base temporale sono nanosecondi,
 quindi "1" è basso. Si consiglia quindi di lanciare i comandi che seguono:
	
	create_clock -name virtual -period 10.000					<--- il periodo è più alto
	set_input_delay -clock virtual -add_delay 0 [all_inputs]
	set_output_delay -clock virtual -add_delay 0 [all_outputs]
	derive_clock_uncertainty
	update_timing_netlist

 Ma questo NON basta: occorre anche lanciare il comando <set_clock_uncertainty>. Nella
 documentazione è riportata la riga di esempio:
 
	set_clock_uncertainty -setup -rise_from clk1 -fall_to clk2 200ps
	
 NON va bene! non so cosa occorra settare esattamente, ma la riga risultante nel file
 .SDC non deve contenere "-setup" e deve essere simili alle altre, ossia avere i due
 campi "rise_from" e "rise_to", come segue:
 
	set_clock_uncertainty -rise_from [get_clocks {virtual}] -rise_to [get_clocks {virtual}] 0.100  
	set_clock_uncertainty -rise_from [get_clocks {altera_reserved_tck}] -rise_to [get_clocks {altera_reserved_tck}]  0.020  
	set_clock_uncertainty -rise_from [get_clocks {altera_reserved_tck}] -fall_to [get_clocks {altera_reserved_tck}]  0.020  

 La FALL_TO io non l'ho messa. Comunque i warning si sono più che dimezzati.
 **************************************************************************************/
 
// TOP MODULE
module main(
	input	clk,
	input	key0,
	input	key1,
	output	[SIZE-1:0]led);

parameter SIZE = 8;
	
wire TDo, TDi;				// Transfer data In/Out
wire [SIZE-1:0] IRi, IRo;	// Instruction register In/Out
wire tck;					// clock virtuale del JTAG
wire vsdr;					// Virtual State Data register
wire [SIZE-1:0]	DataIn;		// registro 0 e registro 1 che verranno moltiplicati

reg [SIZE-1:0]	DataOut = 8'd0;	// registro dati in uscita


// Il vJTAG viene istanziato con una "Explicit Internal Connection" così da non dover
// definire tutti gli input e output
vjtag vJTAG(
	.ir_out(IRo),
	.tdo(TDo),
	.ir_in(IRi),
	.tdi(TDi),
	.tck(tck),
	//.tms(),
	.virtual_state_sdr(vsdr));

// Istanza dello shifter per caricare i dati (NON i comandi)
DataShifterIN #(SIZE)
DS_in
(	.clk(tck),
	.vsdr(vsdr),
	.tdi(TDi),
	.q(DataIn));
	
DataShifterOUT #(SIZE)
DS_out
(	.clk(tck),
	.vsdr(vsdr),
	.data(DataOut),
	.tdo(TDo));

always @(posedge key0)
begin
	DataOut <= DataOut + 1'b1;
end
	
assign led = key1 ? IRi : DataIn;	// Ricorda che le KEY sono in logica negata: premuta è 0

endmodule
